/*
 * Copyright (c) 2000 Blue Mug, Inc.  All Rights Reserved.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if !defined(WIN32)
#include <unistd.h>
#endif

#include "util.h"
#include "log.h"

extern char *progname;

void print_perror(const char *errstr){
	char str[256];
	int ptr = 0;

	ptr = sprintf(&str[0], "%s: ", progname);	
	ptr += sprintf(&str[ptr], "%s: %s\n", errstr, strerror(errno));
	
	message(str);
	messagebox(str);
}

void print_size(unsigned long start, int size){
	char str[256];
  
	if(!size){
		sprintf(str, "- start: 0x%08lx size: 0x%08x\n", start, size);
	}else{
		sprintf(str, "- start: 0x%08lx size: 0x%08x last: 0x%08lx\n",
			start, size, start + (unsigned long)(size - 1));
	}
	message(str);
}

int read_file(const char *filename, void **_buf, int *size){
	FILE *fp;
	char **buf = (char **)_buf;
	int filesize;
	int readsize;
	int bufsize;
	char str[256];

	fp = fopen(filename, "rb");
	if(fp == NULL){
		print_perror(filename);
		return -1;
	}

	fseek(fp, 0L, SEEK_END);
	bufsize = filesize = ftell(fp);
	if(*size > bufsize){	/* min buffer size requested */
		bufsize = *size;
	}	
	if(bufsize & 1){/* round up to even # of bytes for eth loader */
		bufsize += 1;
	}

	*buf = malloc(bufsize);
	if(*buf == NULL){
	  		message("virtual memory exhausted\n");
			return -1;
	}

	fseek(fp, 0L, SEEK_SET);
	readsize = fread(*buf, 1, filesize, fp);
	fclose(fp);
	if(readsize != filesize){
		sprintf(str, "%s: couldn't read whole file\n", filename);
		message(str);
		return -1;
	}

	sprintf(str, "%s: %d bytes (%d bytes buffer)\n",
		filename, filesize, bufsize);
	message(str);

	*size = readsize;
	if(*size & 1){
		*size += 1;
	}
	return 0;
}

#if 0
void *xmalloc(size_t size)
{
	void *value = malloc(size);
	if (!value) {
		message("virtual memory exhausted\n");
		return NULL;
	}
	return value;
}


void xclose(_fd_type fd)
{
#if !defined(WIN32)
	if (close(fd) < 0)
		perror_exit("close");
#else
	if (!CloseHandle(fd))
		perror_exit("close");
#endif
}


/* read or die */
ssize_t xread(_fd_type fd, void *_buf, size_t count)
{
	char *buf = (char *)_buf;
#if !defined(WIN32)
	ssize_t retval;
	do {
		retval = read(fd, buf, count);
	} while ((retval < 0) && (errno == EINTR));
	if (retval < 0)
		perror_exit("read");
	return retval;
#else
	DWORD retval;
	ReadFile(fd, buf, count, &retval, NULL);
	if (retval < 0)
		perror_exit("read");
	return retval;
#endif
}

/* read entire buffer or die */
ssize_t xaread(_fd_type fd, void *_buf, size_t count)
{
	char *buf = (char *)_buf;
	int nread, remain;
	for (remain = count; remain; remain -= nread)
		buf += (nread = xread(fd, buf, remain));
	return count;
}

/* write or die */
ssize_t xwrite(_fd_type fd, const void *_buf, size_t count)
{
	char *buf = (char *)_buf;
#if !defined(WIN32)
	ssize_t retval;
	do {
		retval = write(fd, buf, count);
	} while ((retval < 0) && (errno == EINTR));
	if (retval < 0)
		perror_exit("write");
	return retval;
#else
	DWORD retval;
	WriteFile(fd, buf, count, &retval, NULL);
	if (retval < 0)
		perror_exit("write");
	return retval;
#endif
}

/* write entire buffer or die */
ssize_t xawrite(_fd_type fd, const void *_buf, size_t count)
{
	char *buf = (char *)_buf;
	int nwritten, remain;
	for (remain = count; remain; remain -= nwritten)
		buf += (nwritten = xwrite(fd, buf, remain));
	return count;
}

#endif
